#ifndef COLLECTIONH
#define COLLECTIONH

/* Collection.h -- declarations for abstract Collection class

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

Modification History:

*/

#include "Object.h"

const UNSIGNED CLTN_DEFAULT_CAPACITY	= 16;	// default initial collection capacity 
const UNSIGNED CLTN_EXPANSION_INCREMENT = 32;	// collection (OrderedCltn) expansion increment 
const UNSIGNED CLTN_EXPANSION_FACTOR = 2;	// collection (Set,Bag,Dictionary) expansion factor 

#define DO(cltn,cls,arg)\
{ register cls arg; DoCltnPos DO_pos; while (arg = (cls)((cltn).doNext(DO_pos))) {
#define OD }}

struct DoCltnPos {		/* Collection position for the DO macro */
	int	index;	// used by most Collections 
	void*	ptr;	// used by LinkedLists 
	UNSIGNED num;	// used by Bags 
	DoCltnPos()	{ index = num = 0; ptr = 0; }
};

extern Class class_Collection;

class Arrayobid;
class Bag;
class OrderedCltn;
class Set;
class SortedCltn;

class Collection: public Object {	// abstract class 
public:
	Collection() {}
	Collection(fileDescTy&,Collection&) {}
	Collection(istream&,Collection&) {}
	Arrayobid	asArrayobid();
	Bag		asBag();
	OrderedCltn	asOrderedCltn();
	Set		asSet();
	SortedCltn	asSortedCltn();
	virtual	obid		add(const Object&);
	virtual const Collection& addAll(const Collection&);
	virtual Collection&	addContentsTo(Collection&);
	virtual void		deepenShallowCopy();
	virtual	obid		doNext(DoCltnPos& pos);
	virtual bool		includes(const Object&);
	virtual const Class*	isA();
	virtual bool		isEmpty();
	virtual UNSIGNED	occurrencesOf(const Object&);
	virtual obid		remove(const Object&);
	virtual const Collection& removeAll(const Collection&);
	virtual obid		shallowCopy();		// shouldNotImplement
	virtual UNSIGNED	size();
	virtual void		storer(fileDescTy&);
	virtual void		storer(ostream&);
};

#endif
