/* oopsinit.c -- OOPS initialization

Author:
	K. E. Gorlen
	Bg. 12A, Rm. 2017
	Computer Systems Laboratory
	Division of Computer Research and Technology
	National Institutes of Health
	Bethesda, Maryland 20892
	Phone: (301) 496-5363
	uucp: {decvax!}seismo!elsie!cecil!keith

Function:
	
Initializes OOPS

Modification History:
	
29-Jun-86	K. E. Gorlen

1.  Add OOPS tables.

29-Apr-86	K. E. Gorlen

1.  Add OOPS__Initialized flag.

12-Apr-86	K. E. Gorlen

1.  Remove code for special initialization of main process, now done
with an initor2 function.

*/
#include "Object.h"
#include <errlib.h>

// OOPS Tables

unsigned char	char_bit_mask[sizeof(char)];
unsigned short	short_bit_mask[sizeof(short)];
unsigned int	int_bit_mask[sizeof(int)];
unsigned char bit_count[256];
unsigned char bit_reverse[256];
	
static void InitTables()
{
	register unsigned i,j;
	
	for (i=0, j=1; i<sizeof(char); i++, j <<= 1) char_bit_mask[i] = j;
	for (i=0, j=1; i<sizeof(short); i++, j <<= 1) short_bit_mask[i] = j;
	for (i=0, j=1; i<sizeof(int); i++, j <<= 1) int_bit_mask[i] = j;

	for (i=0; i<256; i++) {
		bit_count[i] = 0;
		j = i;
		while (j != 0) {
			bit_count[i]++;
			j &= j-1;
		}
	}
	
	for (i=0; i<256; i++) {
		bit_reverse[i] = 0;
		j = i;
		register unsigned char m = 0x80;
		while (j != 0) {
			if ((j&1) != 0) bit_reverse[i] |= m;
			j >>= 1;
			m >>= 1;
		}
	}
}

extern void OOPS__errinit();	// error facility initializer for OOPS
extern void OOPS__classInit();	// initialize OOPS classes 

bool OOPS__Initialized;		// YES if OOPS initialization completed

void OOPS_init()		// OOPS initialization, called once from _main 
				// AFTER execution of all static constructors
{
	OOPS__errinit();	// initialize the OOPS error handler 
	seterropt(ERROR,WARNING,NO,3,NULL);
	InitTables();		// initialize OOPS tables
	OOPS__classInit();	// initialize OOPS classes
	OOPS__Initialized = YES;
}
